@@ -1,4 +1,4 @@
-# Copyright (c) 2021  StorPool <support@storpool.com>
+# Copyright (c) 2021, 2022  StorPool <support@storpool.com>
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -36,7 +36,8 @@ import cfg_diag
 import jinja2
 import trivver
 
-from sp_variant import __main__ as variant
+from sp_variant import defs
+from sp_variant import variant
 
 
 @dataclasses.dataclass(frozen=True)
@@ -99,14 +100,10 @@ def regex_un_x(value: str) -> str:
     )
 
 
-def dictvsort(
-    data: Dict[str, variant.Variant]
-) -> List[Tuple[str, variant.Variant]]:
+def dictvsort(data: Dict[str, variant.Variant]) -> List[Tuple[str, variant.Variant]]:
     """Sort a dict of variants by name, preserving some numerical order."""
 
-    def compare(
-        left: Tuple[str, variant.Variant], right: Tuple[str, variant.Variant]
-    ) -> int:
+    def compare(left: Tuple[str, variant.Variant], right: Tuple[str, variant.Variant]) -> int:
         """Compare two variants by name."""
         res = trivver.compare(left[0], right[0])
         assert res != 0
@@ -122,7 +119,7 @@ def vsort(names: List[str]) -> List[str]:
 
 def build_json(var: variant.Variant) -> str:
     """Represent the variant data as a JSON string."""
-    return json.dumps(variant.jsonify(var), sort_keys=True, indent=2)
+    return json.dumps(defs.jsonify(var), sort_keys=True, indent=2)
 
 
 def substitute(cfg: Config) -> None:
@@ -140,12 +137,12 @@ def substitute(cfg: Config) -> None:
     jenv.filters["regexunx"] = regex_un_x
     jenv.filters["vsort"] = vsort
     jvars = {
-        "format_version": variant.FORMAT_VERSION,
+        "format_version": defs.FORMAT_VERSION,
         "order": [var.name for var in variants],
-        "repotypes": {repo.name: repo for repo in variant.REPO_TYPES},
+        "repotypes": {repo.name: repo for repo in defs.REPO_TYPES},
         "variants": {var.name: var for var in variants},
         "variants_json": {var.name: build_json(var) for var in variants},
-        "version": variant.VERSION,
+        "version": defs.VERSION,
     }
 
     cfg.diag("Rendering the template")
