@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2021  StorPool <support@storpool.com>
+ * Copyright (c) 2021, 2022  StorPool <support@storpool.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -32,33 +32,68 @@
 //! - display the OS variant data as a JSON object
 
 #![warn(missing_docs)]
+// Turn on most of the clippy::restriction lints...
+#![warn(clippy::exhaustive_enums)]
+#![warn(clippy::exhaustive_structs)]
+#![warn(clippy::integer_arithmetic)]
+#![warn(clippy::missing_inline_in_public_items)]
+#![warn(clippy::panic)]
+#![warn(clippy::pattern_type_mismatch)]
+#![warn(clippy::shadow_reuse)]
+#![warn(clippy::shadow_unrelated)]
+#![warn(clippy::single_char_lifetime_names)]
+#![warn(clippy::str_to_string)]
+#![warn(clippy::string_slice)]
+#![warn(clippy::string_to_string)]
+#![warn(clippy::unwrap_used)]
+#![warn(clippy::use_debug)]
+#![warn(clippy::verbose_file_reads)]
+// ...except for these ones.
+#![allow(clippy::implicit_return)]
+#![allow(clippy::indexing_slicing)]
+#![allow(clippy::missing_docs_in_private_items)]
+#![allow(clippy::print_stdout)]
+// Also turn on some of the clippy::pedantic lints.
+#![warn(clippy::doc_markdown)]
+#![warn(clippy::implicit_clone)]
+#![warn(clippy::items_after_statements)]
+#![warn(clippy::manual_assert)]
+#![warn(clippy::match_bool)]
+#![warn(clippy::missing_errors_doc)]
+#![warn(clippy::must_use_candidate)]
+#![warn(clippy::needless_pass_by_value)]
+#![warn(clippy::redundant_closure_for_method_calls)]
+#![warn(clippy::similar_names)]
+#![warn(clippy::too_many_lines)]
 
+use std::borrow::ToOwned;
 use std::collections::HashMap;
-use std::fs;
-use std::io::{Read, Write};
+use std::fs::{self, OpenOptions};
+use std::io::Write;
 use std::os::unix::fs::PermissionsExt;
 use std::os::unix::io::AsRawFd;
 use std::os::unix::process::ExitStatusExt;
-use std::path;
-use std::process;
+use std::path::Path;
+use std::process::Command;
 
-use expect_exit::ExpectedWithError;
-use nix::unistd;
-use serde::{Deserialize, Serialize};
+use clap::{App, Arg, ArgMatches, SubCommand};
+use expect_exit::{ExpectedResult, ExpectedWithError};
+use nix::unistd::{self, Gid, Uid};
+use serde_json::json;
 
-use sp_variant::{self, Variant, VariantDefTop};
+use sp_variant::{self, DebRepo, Repo, Variant, VariantDefTop, YumRepo};
 
 #[derive(Debug)]
-struct RepoType<'a> {
-    name: &'a str,
-    extension: &'a str,
+struct RepoType<'data> {
+    name: &'data str,
+    extension: &'data str,
 }
 
 #[derive(Debug)]
-struct RepoAddConfig<'a> {
+struct RepoAddConfig<'data> {
     noop: bool,
     repodir: String,
-    repotype: &'a RepoType<'a>,
+    repotype: &'data RepoType<'data>,
 }
 
 #[derive(Debug)]
@@ -74,20 +109,13 @@ struct ShowConfig {
     name: String,
 }
 
-#[derive(Debug, Serialize, Deserialize)]
-struct SingleVariant {
-    format: sp_variant::VariantFormat,
-    variant: Variant,
-    version: String,
-}
-
 #[derive(Debug)]
-enum Mode<'a> {
+enum Mode<'data> {
     CommandList,
     CommandRun(CommandRunConfig),
     Detect,
     Features,
-    RepoAdd(RepoAddConfig<'a>),
+    RepoAdd(RepoAddConfig<'data>),
     Show(ShowConfig),
 }
 
@@ -131,7 +159,7 @@ fn run_command(cmdvec: &[String], action: &str, noop: bool) {
         return;
     }
 
-    let status = process::Command::new(&cmdvec[0])
+    let status = Command::new(&cmdvec[0])
         .args(&cmdvec[1..])
         .spawn()
         .or_exit_e(|| format!("{}: {}", action, cmdstr))
@@ -158,20 +186,12 @@ fn run_command(cmdvec: &[String], action: &str, noop: bool) {
 fn copy_file(fname: &str, srcdir: &str, dstdir: &str, noop: bool) {
     let src = format!("{}/{}", srcdir, fname);
     let dst = format!("{}/{}", dstdir, fname);
-    println!("Copying {:?} -> {:?}", src, dst);
+    println!("Copying {} -> {}", src, dst);
 
-    let read_source_file = || {
-        let mut infile =
-            fs::File::open(&src).or_exit_e(|| format!("Could not open {} for reading", src));
-        let mut contents = Vec::<u8>::new();
-        infile
-            .read_to_end(&mut contents)
-            .or_exit_e(|| format!("Could not read from {}", src));
-        contents
-    };
+    let read_source_file = || fs::read(&src).or_exit_e(|| format!("Could not read from {}", src));
 
     let write_destination_file = |contents: &Vec<u8>| {
-        let mut outfile = fs::OpenOptions::new()
+        let mut outfile = OpenOptions::new()
             .write(true)
             .create(true)
             .truncate(true)
@@ -187,8 +207,8 @@ fn copy_file(fname: &str, srcdir: &str, dstdir: &str, noop: bool) {
             .or_exit_e(|| format!("Could not change the mode on {}", dst));
         unistd::fchown(
             outfile.as_raw_fd(),
-            Some(unistd::Uid::from_raw(0)),
-            Some(unistd::Gid::from_raw(0)),
+            Some(Uid::from_raw(0)),
+            Some(Gid::from_raw(0)),
         )
         .or_exit_e(|| format!("Could not set the ownership of {}", dst));
         outfile
@@ -206,10 +226,29 @@ fn copy_file(fname: &str, srcdir: &str, dstdir: &str, noop: bool) {
     write_destination_file(&contents);
 }
 
-fn repo_add_deb(var: &Variant, config: RepoAddConfig, vdir: &str, repo: &sp_variant::DebRepo) {
+fn get_filename<'path>(path: &'path str, tag: &str) -> &'path str {
+    path.rsplit('/')
+        .next()
+        .expect_result(|| format!("Could not obtain a filename from '{}'", path))
+        .or_exit_e(|| format!("Internal error: could not parse a {}", tag))
+}
+
+fn get_filename_extension<'fname>(filename: &'fname str, tag: &str) -> (&'fname str, &'fname str) {
+    filename
+        .rsplit_once('.')
+        .expect_result(|| {
+            format!(
+                "Could not split '{}' into a filename and extension",
+                filename
+            )
+        })
+        .or_exit_e(|| format!("Internal error: could not parse a {}", tag))
+}
+
+fn repo_add_deb(var: &Variant, config: &RepoAddConfig, vdir: &str, repo: &DebRepo) {
     let install_req_packages = || {
         // First, install the ca-certificates package if required...
-        let mut cmdvec: Vec<String> = var.commands["package"]["install"].to_vec();
+        let mut cmdvec: Vec<String> = var.commands["package"]["install"].clone();
         cmdvec.extend(repo.req_packages.iter().cloned());
         run_command(
             &cmdvec,
@@ -219,8 +258,8 @@ fn repo_add_deb(var: &Variant, config: RepoAddConfig, vdir: &str, repo: &sp_vari
     };
 
     let copy_sources_file = || {
-        let sources_orig = repo.sources.rsplit('/').next().unwrap();
-        let (sources_base, sources_ext) = sources_orig.rsplit_once('.').unwrap();
+        let sources_orig = get_filename(&repo.sources, "Apt sources list");
+        let (sources_base, sources_ext) = get_filename_extension(sources_orig, "Apt sources list");
         let sources_fname = format!(
             "{}{}.{}",
             sources_base, config.repotype.extension, sources_ext
@@ -229,13 +268,13 @@ fn repo_add_deb(var: &Variant, config: RepoAddConfig, vdir: &str, repo: &sp_vari
     };
 
     let copy_keyring_file = || {
-        let keyring_fname = repo.keyring.rsplit('/').next().unwrap();
+        let keyring_fname = get_filename(&repo.keyring, "Apt keyring");
         copy_file(keyring_fname, vdir, "/usr/share/keyrings", config.noop);
     };
 
     let run_apt_update = || {
         run_command(
-            &["apt-get".to_string(), "update".to_string()],
+            &["apt-get".to_owned(), "update".to_owned()],
             "Could not update the package database",
             config.noop,
         );
@@ -250,16 +289,16 @@ fn repo_add_deb(var: &Variant, config: RepoAddConfig, vdir: &str, repo: &sp_vari
     run_apt_update();
 }
 
-fn repo_add_yum(config: RepoAddConfig, vdir: &str, repo: &sp_variant::YumRepo) {
+fn repo_add_yum(config: &RepoAddConfig, vdir: &str, repo: &YumRepo) {
     let run_yum_install_certs = || {
         run_command(
             &[
-                "yum".to_string(),
-                "--disablerepo=storpool-*".to_string(),
-                "install".to_string(),
-                "-q".to_string(),
-                "-y".to_string(),
-                "ca-certificates".to_string(),
+                "yum".to_owned(),
+                "--disablerepo=storpool-*".to_owned(),
+                "install".to_owned(),
+                "-q".to_owned(),
+                "-y".to_owned(),
+                "ca-certificates".to_owned(),
             ],
             "Could not update the package database",
             config.noop,
@@ -267,8 +306,9 @@ fn repo_add_yum(config: RepoAddConfig, vdir: &str, repo: &sp_variant::YumRepo) {
     };
 
     let copy_yumdef_file = || {
-        let yumdef_orig = repo.yumdef.rsplit('/').next().unwrap();
-        let (yumdef_base, yumdef_ext) = yumdef_orig.rsplit_once('.').unwrap();
+        let yumdef_orig = get_filename(&repo.yumdef, "Yum repository definition");
+        let (yumdef_base, yumdef_ext) =
+            get_filename_extension(yumdef_orig, "Yum repository definition");
         let yumdef_fname = format!(
             "{}{}.{}",
             yumdef_base, config.repotype.extension, yumdef_ext
@@ -276,21 +316,18 @@ fn repo_add_yum(config: RepoAddConfig, vdir: &str, repo: &sp_variant::YumRepo) {
         copy_file(&yumdef_fname, vdir, "/etc/yum.repos.d", config.noop);
     };
 
+    let keyring_fname = get_filename(&repo.keyring, "Yum keyring");
     let copy_keyring_file = || {
-        let keyring_fname = repo.keyring.rsplit('/').next().unwrap();
         copy_file(keyring_fname, vdir, "/etc/pki/rpm-gpg", config.noop);
     };
 
     let run_rpmkeys = || {
-        if path::Path::new("/usr/bin/rpmkeys").exists() {
+        if Path::new("/usr/bin/rpmkeys").exists() {
             run_command(
                 &[
-                    "rpmkeys".to_string(),
-                    "--import".to_string(),
-                    format!(
-                        "/etc/pki/rpm-gpg/{}",
-                        repo.keyring.rsplit('/').next().unwrap()
-                    ),
+                    "rpmkeys".to_owned(),
+                    "--import".to_owned(),
+                    format!("/etc/pki/rpm-gpg/{}", keyring_fname),
                 ],
                 "Could not import the StorPool RPM OpenPGP keys",
                 config.noop,
@@ -301,11 +338,11 @@ fn repo_add_yum(config: RepoAddConfig, vdir: &str, repo: &sp_variant::YumRepo) {
     let run_yum_clean_metadata = || {
         run_command(
             &[
-                "yum".to_string(),
-                "--disablerepo=*".to_string(),
+                "yum".to_owned(),
+                "--disablerepo=*".to_owned(),
                 format!("--enablerepo=storpool-{}", config.repotype.name),
-                "clean".to_string(),
-                "metadata".to_string(),
+                "clean".to_owned(),
+                "metadata".to_owned(),
             ],
             "Could not update the package database",
             config.noop,
@@ -319,7 +356,7 @@ fn repo_add_yum(config: RepoAddConfig, vdir: &str, repo: &sp_variant::YumRepo) {
     run_yum_clean_metadata();
 }
 
-fn cmd_repo_add(varfull: &VariantDefTop, config: RepoAddConfig) {
+fn cmd_repo_add(varfull: &VariantDefTop, config: &RepoAddConfig) {
     let var = detect_variant(varfull);
     let vdir = format!("{}/{}", config.repodir, var.kind.as_ref());
     if !fs::metadata(&vdir)
@@ -328,9 +365,10 @@ fn cmd_repo_add(varfull: &VariantDefTop, config: RepoAddConfig) {
     {
         expect_exit::die(&format!("Not a directory: {:?}", vdir));
     }
-    match &var.repo {
-        sp_variant::Repo::Deb(deb) => repo_add_deb(var, config, &vdir, deb),
-        sp_variant::Repo::Yum(yum) => repo_add_yum(config, &vdir, yum),
+    match var.repo {
+        Repo::Deb(ref deb) => repo_add_deb(var, config, &vdir, deb),
+        Repo::Yum(ref yum) => repo_add_yum(config, &vdir, yum),
+        _ => expect_exit::exit("Internal error: unhandled repo type"),
     }
 }
 
@@ -340,7 +378,7 @@ fn cmd_command_list(varfull: &VariantDefTop) {
         K: Ord,
     {
         let mut res: Vec<_> = map.iter().collect();
-        res.sort_by_key(|(key, _)| *key);
+        res.sort_by_key(|&(key, _)| key);
         res
     }
 
@@ -360,7 +398,7 @@ fn cmd_command_run(varfull: &VariantDefTop, config: CommandRunConfig) {
     let var = detect_variant(varfull);
     let mut cmd_vec: Vec<String> = match var.commands.get(&config.category) {
         Some(cmap) => match cmap.get(&config.name) {
-            Some(cmd) => cmd.to_vec(),
+            Some(cmd) => cmd.clone(),
             None => expect_exit::exit("Unknown command identifier"),
         },
         None => expect_exit::exit("Unknown command identifier"),
@@ -369,62 +407,86 @@ fn cmd_command_run(varfull: &VariantDefTop, config: CommandRunConfig) {
     run_command(&cmd_vec, "Command failed", config.noop);
 }
 
-fn cmd_show(varfull: &VariantDefTop, config: ShowConfig) {
-    match config.name == "all" {
-        true => print!("{}", serde_json::to_string(varfull).unwrap()),
-        false => {
-            let var = match &*config.name {
-                "current" => {
-                    sp_variant::detect_from(varfull).or_exit_e_("Cannot detect the current variant")
-                }
-                other => sp_variant::get_from(varfull, other).or_exit_e_("Invalid variant name"),
-            };
-            let (major, minor) = sp_variant::get_format_version_from(varfull);
-            let single = SingleVariant {
-                format: sp_variant::VariantFormat {
-                    version: sp_variant::VariantFormatVersion { major, minor },
+fn cmd_show(varfull: &VariantDefTop, config: &ShowConfig) {
+    if config.name == "all" {
+        print!(
+            "{}",
+            serde_json::to_string(varfull)
+                .or_exit_e_("Internal error: could not serialize the variant data")
+        );
+    } else {
+        let var = match &*config.name {
+            "current" => {
+                sp_variant::detect_from(varfull).or_exit_e_("Cannot detect the current variant")
+            }
+            other => sp_variant::get_from(varfull, other).or_exit_e_("Invalid variant name"),
+        };
+        let (major, minor) = sp_variant::get_format_version_from(varfull);
+        let single = json!({
+            "format": {
+                "version": {
+                    "major": major,
+                    "minor": minor,
                 },
-                variant: var.clone(),
-                version: sp_variant::get_program_version().to_string(),
-            };
-            println!("{}", serde_json::to_string_pretty(&single).unwrap());
-        }
-    };
+            },
+            "variant": var.clone(),
+            "version": sp_variant::get_program_version().to_owned(),
+        });
+        println!(
+            "{}",
+            serde_json::to_string_pretty(&single)
+                .or_exit_e_("Internal error: could not serialize the variant data")
+        );
+    }
 }
 
+// This one could probably go away with a clap 3 refactoring... maybe.
+#[allow(clippy::too_many_lines)]
 fn main() {
+    type Handler<'cmds> = &'cmds dyn Fn(&'cmds ArgMatches) -> Mode<'cmds>;
+
+    fn get_subc_name<'cmds>(current: &'cmds SubCommand) -> (String, &'cmds ArgMatches<'cmds>) {
+        match current.matches.subcommand {
+            Some(ref next) => {
+                let (next_name, matches) = get_subc_name(next);
+                (format!("{}/{}", current.name, next_name), matches)
+            }
+            None => (current.name.clone(), &current.matches),
+        }
+    }
+
     let varfull = sp_variant::build_variants();
     let program_version = sp_variant::get_program_version_from(varfull);
     let app = {
         let valid_repo_types: Vec<&str> = REPO_TYPES.iter().map(|rtype| rtype.name).collect();
-        clap::App::new("storpool_variant")
+        App::new("storpool_variant")
             .version(program_version)
             .author("StorPool <support@storpool.com>")
             .about("storpool_variant: handle OS distribution- and version-specific tasks")
             .subcommand(
-                clap::SubCommand::with_name("command")
+                SubCommand::with_name("command")
                     .about("Distribition-specific commands")
                     .subcommand(
-                        clap::SubCommand::with_name("list")
+                        SubCommand::with_name("list")
                             .about("List the distribution-specific commands"),
                     )
                     .subcommand(
-                        clap::SubCommand::with_name("run")
+                        SubCommand::with_name("run")
                             .about("Run a distribution-specific command")
                             .arg(
-                                clap::Arg::with_name("noop")
+                                Arg::with_name("noop")
                                     .short("N")
                                     .long("noop")
                                     .help("No-operation mode; display what would be done"),
                             )
                             .arg(
-                                clap::Arg::with_name("command")
+                                Arg::with_name("command")
                                     .index(1)
                                     .required(true)
                                     .help("The identifier of the command to run"),
                             )
                             .arg(
-                                clap::Arg::with_name("args")
+                                Arg::with_name("args")
                                     .index(2)
                                     .multiple(true)
                                     .help("Arguments to pass to the command"),
@@ -432,27 +494,27 @@ fn main() {
                     ),
             )
             .subcommand(
-                clap::SubCommand::with_name("detect")
+                SubCommand::with_name("detect")
                     .about("Detect the build variant for the current host"),
             )
             .subcommand(
-                clap::SubCommand::with_name("features")
+                SubCommand::with_name("features")
                     .about("Display the features supported by storpool_variant"),
             )
             .subcommand(
-                clap::SubCommand::with_name("repo")
+                SubCommand::with_name("repo")
                     .about("StorPool repository-related commands")
                     .subcommand(
-                        clap::SubCommand::with_name("add")
+                        SubCommand::with_name("add")
                             .about("Install the StorPool repository configuration")
                             .arg(
-                                clap::Arg::with_name("noop")
+                                Arg::with_name("noop")
                                     .short("N")
                                     .long("noop")
                                     .help("No-operation mode; display what would be done"),
                             )
                             .arg(
-                                clap::Arg::with_name("repodir")
+                                Arg::with_name("repodir")
                                     .short("d")
                                     .required(true)
                                     .takes_value(true)
@@ -460,7 +522,7 @@ fn main() {
                                     .help("The path to the repo config directory"),
                             )
                             .arg(
-                                clap::Arg::with_name("repotype")
+                                Arg::with_name("repotype")
                                     .short("t")
                                     .takes_value(true)
                                     .value_name("REPOTYPE")
@@ -471,39 +533,29 @@ fn main() {
                     ),
             )
             .subcommand(
-                clap::SubCommand::with_name("show")
+                SubCommand::with_name("show")
                     .about("Display information about a build variant")
                     .arg(
-                        clap::Arg::with_name("name")
+                        Arg::with_name("name")
                             .index(1)
                             .required(true)
                             .help("the name of the build variant to query"),
                     ),
             )
     };
-    let matches = app.get_matches();
+    let opt_matches = app.get_matches();
 
-    fn get_subc_name<'a>(current: &'a clap::SubCommand) -> (String, &'a clap::ArgMatches<'a>) {
-        match &current.matches.subcommand {
-            Some(next) => {
-                let (next_name, matches) = get_subc_name(next);
-                (format!("{}/{}", current.name, next_name), matches)
-            }
-            None => (current.name.to_string(), &current.matches),
-        }
-    }
-
-    type Handler<'a> = &'a dyn Fn(&'a clap::ArgMatches) -> Mode<'a>;
+    #[allow(clippy::unwrap_used)]
     let cmds: Vec<(&str, Handler)> = vec![
         ("command/list", &|_matches| Mode::CommandList),
         ("command/run", &|matches| {
             let parts: Vec<&str> = matches.value_of("command").unwrap().split('.').collect();
             match parts.len() {
                 2 => Mode::CommandRun(CommandRunConfig {
-                    category: parts[0].to_string(),
-                    name: parts[1].to_string(),
+                    category: parts[0].to_owned(),
+                    name: parts[1].to_owned(),
                     args: match matches.values_of("args") {
-                        Some(args) => args.map(|value| value.to_string()).collect(),
+                        Some(args) => args.map(ToOwned::to_owned).collect(),
                         None => vec![],
                     },
                     noop: matches.is_present("noop"),
@@ -516,7 +568,7 @@ fn main() {
         ("repo/add", &|matches| {
             Mode::RepoAdd(RepoAddConfig {
                 noop: matches.is_present("noop"),
-                repodir: matches.value_of("repodir").unwrap().to_string(),
+                repodir: matches.value_of("repodir").unwrap().to_owned(),
                 repotype: {
                     let name = matches.value_of("repotype").unwrap();
                     REPO_TYPES.iter().find(|rtype| rtype.name == name).unwrap()
@@ -525,28 +577,28 @@ fn main() {
         }),
         ("show", &|matches| {
             Mode::Show(ShowConfig {
-                name: matches.value_of("name").unwrap().to_string(),
+                name: matches.value_of("name").unwrap().to_owned(),
             })
         }),
     ];
-    match &matches.subcommand {
-        Some(subcommand) => {
+    match opt_matches.subcommand {
+        Some(ref subcommand) => {
             let (subc_name, subc_matches) = get_subc_name(subcommand);
             match cmds
                 .iter()
-                .find_map(|(name, handler)| (*name == subc_name).then(|| handler))
+                .find_map(|&(name, handler)| (*name == subc_name).then(|| handler))
             {
                 Some(handler) => match handler(subc_matches) {
                     Mode::Features => cmd_features(varfull),
                     Mode::CommandList => cmd_command_list(varfull),
                     Mode::CommandRun(config) => cmd_command_run(varfull, config),
                     Mode::Detect => cmd_detect(varfull),
-                    Mode::RepoAdd(config) => cmd_repo_add(varfull, config),
-                    Mode::Show(config) => cmd_show(varfull, config),
+                    Mode::RepoAdd(config) => cmd_repo_add(varfull, &config),
+                    Mode::Show(config) => cmd_show(varfull, &config),
                 },
-                None => expect_exit::exit(matches.usage()),
+                None => expect_exit::exit(opt_matches.usage()),
             }
         }
-        None => expect_exit::exit(matches.usage()),
+        None => expect_exit::exit(opt_matches.usage()),
     }
 }
